Cargo features of the ialloc crate.
| Feature | Description | Additional Requirements |
|---|---|---|
| API Design | ||
"panicy“ | Implicitly panicy APIs | |
"panicy-bounds" | APIs that panic on out of bounds indicies | |
"panicy-memory" | APIs that panic when out of memory | |
| (always) | APIs that try to panic on undefined behavior | |
| Dependencies | ||
"alloc" | alloc crate support (rust standard library) | |
"std" | std crate support (rust standard library) | |
"msvc" | MSVC-specific library support | target_env = |
"win32" | Windows-specific allocators | target_os = |
| Language Standards | ||
"c89" | C89 standard library support | cc finds a C89+ compatible compiler |
"c11" | C11 standard library support | cc finds a C11+ compatible compiler |
"c23" | C23 standard library support | cc finds a C23+ compatible compiler |
"c++98" | C++98 standard library support | cc finds a C++98+ compatible compiler |
"c++17" | C++17 standard library support | cc finds a C++17+ compatible compiler |