Module error

Module error 

Structs

BannedZeroSizedAllocationsError
The allocator explicitly rejected a zero-sized allocation, because they behave inconsistently.
ExcessiveAlignmentRequestedError
More alignment was requested than the allocator could support.
ExcessiveSliceRequestedError
A slice large enough to result in a LayoutError was requested.