Alignment

Struct Alignment 

pub struct Alignment(/* private fields */);
A valid Layout alignment (a power of 2)

Available in named constant forms, which have been #[doc(hidden)] to avoid spam:

MinMaxNameEquivalences
ALIGN_1ALIGN_8192Byte(s)8 bits = 1 octect ≈ 1 byte
ALIGN_1_BALIGN_8192_BByte(s)8 bits = 1 octect ≈ 1 byte
ALIGN_1_KiBALIGN_8192_KiBKibibyte(s)1 KiB = 210 bytes = 10241 bytes
ALIGN_1_MiBALIGN_8192_MiBMebibyte(s)1 MiB = 220 bytes = 10242 bytes
ALIGN_1_GiBALIGN_8192_GiBGibibyte(s)1 GiB = 230 bytes = 10243 bytes
ALIGN_1_TiBALIGN_8192_TiBTebibyte(s)1 TiB = 240 bytes = 10244 bytes
ALIGN_1_EiBALIGN_8192_EiBExbibyte(s)1 EiB = 250 bytes = 10245 bytes

Implementations

Source

impl Alignment

pub const MIN: Alignment = ALIGN_1

Minimum representable alignment (e.g. 1)

pub const MAX: Alignment

Maximum representable alignment

BitsMAX
16215 B = 32 KiB
32231 B = 2 GiB
64263 B = 8 EiB
1282127 B = ???
pub const fn new(align: usize) -> Option<Self>

Returns None unless align is a valid power of 2 (which also implies nonzero)

Examples found in repository?
examples/alloc-tester.rs (line 36)
28    pub fn thin(&mut self) -> &mut Self where A : thin::Alloc + thin::Free {
29        let mut thin = AlignmentRange { min: Alignment::MAX, max: Alignment::MAX };
30        for (dst, min_size) in [(&mut thin.min, 1), (&mut thin.max, 4096)] {
31            for _ in 0 .. 100 {
32                let alloc = (self.create)();
33                let addrs = [(); 4096].into_iter().enumerate().map(|(i, _)| alloc.alloc_uninit(i%16+min_size).unwrap()).collect::<Vec<_>>();
34                let addrbits = addrs[..].iter().copied().map(|addr| addr.as_ptr() as usize).reduce(|x,y| x|y).unwrap();
35                addrs.iter().copied().for_each(|addr| unsafe { alloc.free(addr) });
36                let align = Alignment::new(1 << addrbits.trailing_zeros()).unwrap();
37                *dst = align.min(*dst);
38            }
39        }
40        self.thin = Some(thin);
41        self
42    }
43
44    pub fn fat(&mut self) -> &mut Self where A : fat::Alloc + fat::Free {
45        let mut fat = AlignmentRange { min: Alignment::MAX, max: ALIGN_1 };
46
47        for _ in 0 .. 100 {
48            let layout = Layout::new::<u8>(); // 1B
49            let alloc = (self.create)();
50            let addrs = [(); 4096].map(|_| alloc.alloc_uninit(layout).unwrap());
51            let addrbits = addrs[..].iter().copied().map(|addr| addr.as_ptr() as usize).reduce(|x,y| x|y).unwrap();
52            addrs.iter().copied().for_each(|addr| unsafe { alloc.free(addr, layout) });
53            let align = Alignment::new(1 << addrbits.trailing_zeros()).unwrap();
54            fat.min = align.min(fat.min);
55        }
56
57        fat.max = fat.min;
58        let alloc = (self.create)();
59        while let Some(next) = fat.max.as_usize().checked_shl(1) {
60            let Some(align) = Alignment::new(next) else { break };
61            let Ok(layout) = Layout::from_size_align(next, next) else { break };
62            let Ok(addr) = alloc.alloc_uninit(layout) else { break };
63            fat.max = align;
64            unsafe { alloc.free(addr, layout) };
65        }
66
67        self.fat = Some(fat);
68        self
69    }
pub const fn of<T>() -> Self

Returns the Alignment of T.

pub const unsafe fn new_unchecked(align: usize) -> Self

Undefined behavior unless align is a valid power of 2 (which also implies nonzero)

pub const fn as_usize(self) -> usize

Returns the alignment as a usize (the nicheless underlying type)

Examples found in repository?
examples/alloc-tester.rs (line 59)
44    pub fn fat(&mut self) -> &mut Self where A : fat::Alloc + fat::Free {
45        let mut fat = AlignmentRange { min: Alignment::MAX, max: ALIGN_1 };
46
47        for _ in 0 .. 100 {
48            let layout = Layout::new::<u8>(); // 1B
49            let alloc = (self.create)();
50            let addrs = [(); 4096].map(|_| alloc.alloc_uninit(layout).unwrap());
51            let addrbits = addrs[..].iter().copied().map(|addr| addr.as_ptr() as usize).reduce(|x,y| x|y).unwrap();
52            addrs.iter().copied().for_each(|addr| unsafe { alloc.free(addr, layout) });
53            let align = Alignment::new(1 << addrbits.trailing_zeros()).unwrap();
54            fat.min = align.min(fat.min);
55        }
56
57        fat.max = fat.min;
58        let alloc = (self.create)();
59        while let Some(next) = fat.max.as_usize().checked_shl(1) {
60            let Some(align) = Alignment::new(next) else { break };
61            let Ok(layout) = Layout::from_size_align(next, next) else { break };
62            let Ok(addr) = alloc.alloc_uninit(layout) else { break };
63            fat.max = align;
64            unsafe { alloc.free(addr, layout) };
65        }
66
67        self.fat = Some(fat);
68        self
69    }
pub const fn as_nonzero(self) -> NonZeroUsize

Returns the alignment as a NonZeroUsize